/*
 * Copyright 2012 Adrian Toncean
 * 
 * This file is part of From Under the Rock.
 *
 * From Under the Rock is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * From Under the Rock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with From Under the Rock. If not, see <http://www.gnu.org/licenses/>.
 */

package pack

import org.lwjgl._
import opengl.{Display,GL11,DisplayMode}
import GL11._
import input._
import math._
import rooms._
import ds._
import org.lwjgl.util.glu.GLU
import Array._
import org.lwjgl.openal.AL

object Dis {
    val FRAMERATE = 50
 
    var finished = false
    var rx = 0d
    var ry = 0d
    var rz = 0d
    
    val l = 39
    
    var p = new Pos(0,35,0)
    
    var alpha = 0d
    
    var activeRoomX = 0; var activeRoomY = 0
    var nextRoomX = 0; var nextRoomY = 0
    val nmxRoomX = 4; val nmxRoomY = 4
    
    var room = Array.ofDim[Room](nmxRoomX,nmxRoomY)
    
    var cubes_collected = 0
    val cubes_needed = 11

    def main(args:Array[String]) {
        var fullscreen = false
        
        for(arg <- args) {
            arg match{
                case "-fullscreen" =>
                    fullscreen = true
            }
        }

        init(fullscreen)
        initRooms
        Sounds.load
        Sounds.start
        run
        cleanup
    }
    
    def init(fullscreen:Boolean) {
        val width = Display.getDisplayMode.getWidth
        val height = Display.getDisplayMode.getHeight

        println("Init Display")
        Display.setTitle("From Under the rock")
        Display.setFullscreen(fullscreen)
        //Display.setVSyncEnabled(true)
        Display.create
        Display.setVSyncEnabled(!fullscreen)
        //Display.setDisplayMode(new DisplayMode(width,height))
        
        GL11.glViewport(0,0,width,height);
        
        Mouse.setGrabbed(true)

        println("Init GL")
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
        glEnable(GL_BLEND);
        glEnable(GL_DEPTH_TEST);
        adjustCamera
        
        GL11.glColor3f(0f, 0f, 0f)
        GL11.glLineWidth(1)
        
        assemblePrimitives
    }

    def adjustCamera {
        val v = Display.getDisplayMode.getWidth.toFloat/Display.getDisplayMode.getHeight.toFloat
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity
        GLU.gluPerspective(90,v,0.1f,150)
        glMatrixMode(GL_MODELVIEW)
    }

    def cleanup = {
      releasePrimitives
      Display.destroy
      AL.destroy
    }

    def run = {  
      while(!finished) {
        State.getState match {
          case State.PLAY =>
            adv
            input
            checkDoors
            checkItem
          case State.FADEIN =>
            alpha += 0.05
            if(alpha > 1) {
              alpha = 1
              activeRoomX = nextRoomX
              activeRoomY = nextRoomY
              p.z *= -1
              p.x *= -1
              p.z -= p.z/10
              p.x -= p.x/10
              State.setState(State.FADEOUT)
            }
          case State.FADEOUT =>
            alpha -= 0.05 
            if(alpha < 0) {
              alpha = 0
              State.setState(State.PLAY)
            }
          case State.WON =>
            alpha += 0.005
            if(alpha > 1) finished = true 
        }
       
       render
       Display.update
       
       Sounds.adv

       Display.sync(FRAMERATE)
      }
    }
    
    def adv = {
      room(activeRoomX)(activeRoomY).adv
    }
    
    def checkItem = {
      if(room(activeRoomX)(activeRoomY).checkItem(p)) {
        cubes_collected += 1
        if(cubes_collected >= cubes_needed) State.setState(State.WON)
      }
    }
    
    def checkDoors = {
      if(room(activeRoomX)(activeRoomY).isAtDoor(p)) {
        room(activeRoomX)(activeRoomY).getNearestDoor(p) match {
          case 0 => 
            nextRoomY = activeRoomY + nmxRoomY-1
            nextRoomY %= nmxRoomY
          case 1 =>
            nextRoomX = activeRoomX + nmxRoomX-1
            nextRoomX %= nmxRoomX
          case 2 =>
            nextRoomY = activeRoomY + 1
            nextRoomY %= nmxRoomY
          case 3 =>
            nextRoomX = activeRoomX + 1
            nextRoomX %= nmxRoomX
        }
        
        State.setState(State.FADEIN)
      }
    }

    def incPos(k: Double) = {
      p.z += math.cos(math.toRadians(k)) * 0.12;
      p.x += math.sin(math.toRadians(k)) * 0.12;
      
      if(p.z < -l) p.z = -l
      else if(p.z > l) p.z = l
      
      if(p.x < -l) p.x = -l
      else if(p.x > l) p.x = l
    }
    
    def input() {
        import Keyboard._
        
        if(isKeyDown(KEY_ESCAPE))
            finished = true
        if(Display.isCloseRequested)
            finished = true
  
        if(isKeyDown(KEY_W)) {
         incPos(ry)  
        }
        else if(isKeyDown(KEY_S)) {
         incPos(ry+180)
        }
        
        if(isKeyDown(KEY_A)) {
          incPos(ry+90)
        }
        else if(isKeyDown(KEY_D)) {
          incPos(ry-90)
        }
        
        ry -= Mouse.getDX()/4f
        rx -= Mouse.getDY()/4f
    }
    
    def initRooms = {
      room(0)(0) = new RoomForest1
      room(0)(1) = new RoomOct1
      room(0)(2) = new RoomCoral1
      room(0)(3) = new RoomCog1

      room(1)(0) = new RoomA1
      room(1)(1) = new RoomFly1
      room(1)(2) = new RoomForest2
      room(1)(3) = new RoomSnow1
      
      room(2)(0) = new RoomTiles1
      room(2)(1) = new RoomForest3
      room(2)(2) = new RoomSurf1
      room(2)(3) = new RoomDisk1
      
      room(3)(0) = new RoomForest3
      room(3)(1) = new RoomPlant1
      room(3)(2) = new RoomA2
      room(3)(3) = new RoomForest1
    }

    def fadeQuad() {
      glLoadIdentity 
      glBegin(GL_QUADS);
        glColor4f(0.0f,0.0f,0.0f,alpha.toFloat);
        glVertex3f( 3f, 3f,-1.0f);
        glVertex3f(-3f, 3f,-1.0f);
        glVertex3f(-3f,-3f,-1.0f);
        glVertex3f( 3f,-3f,-1.0f);
      glEnd();
      glFlush();
    }
    
    def render() {
        glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
        glLoadIdentity
                
        glRotatef(rx.toFloat,1,0,0)
        glRotatef(-ry.toFloat,0,1,0)
        //glRotatef(rz.toFloat,0,0,1)
        
        glTranslated(p.x,p.y,p.z)
        
        room(activeRoomX)(activeRoomY).draw
        
        if(State.getState != State.PLAY) fadeQuad();
    }
    
    def assemblePrimitives = {
      Box.assemble
      Cube.assemble
      Room.assemble
      Door.assemble
      Snow.assemble
      Square.assemble
      Tiles.assemble
      Coral.assemble
      RedCube.assemble
    }
    
    def releasePrimitives = {
      DSManager.release
    }
}